/**********************************************************************
  Filename    : Camera Remote Bluetooth IoT
  Description : Control android app Camera Remote Bluetooth remotely.
                Change camera modes: photo or video.
                Change flash modes: torch or auto
                Record video.
                Take photo.
  Auther      : www.busywww.com   busyweb@gmail.com
  Modification: 2021/09/09
**********************************************************************/
#include "soc/soc.h"
#include "soc/rtc_cntl_reg.h"

#include "BluetoothSerial.h"
#include "string.h"

#define JX_PHOTO_VIDEO      13     // Joystick X:  photo and video mode
#define JY_FLASH            14     // Joystick Y:  flash on / off

#define PIN_BUTTON_PHOTO    25            // photo mode
#define PIN_BUTTON_VIDEO    27            // video mode
#define PIN_BUTTON_FLASH    33            // flash mode

#define BUTTON_ACTION       32            // Action Button: camera action: either take photo or record video according to current mode
#define BUTTON_CHANGE_CAMERA    26        // change camera: front or back cameras

#define PIN_LED_PHOTO       23            // photo mode led
#define PIN_LED_VIDEO       22            // video mode led
//#define PIN_LED_FLASH       26          // flash mode led
//#define PIN_LED_ACTION      27          // action led

#define PIN_APP_MODE        21            // app mode: joystick or buttons mode

int APP_MODE = 0;                         //0: Switched Mode, 1: Joystick Mode
long modeChangedTime = -1;
long currentTime;

int ACTION_STATUS = 0;                    //0: nothing, 1: action started, 2: action(video) processing
int MODE_PHOTO = 1;                       //1: photo mode, 0: video mode
int MODE_VIDEO = 0;                       //1: video mode, 0: photo mode
int MODE_VIDEO_RECORDING = 0;             // recording video led flashing
int MODE_FLASH = 0;                       // 1: torch flash, 0: auto
int FLASH_STATUS = 0;                     // 1: torch flash, 0: auto


// Bluetooth Status
int CONNECTED = 0;

BluetoothSerial SerialBT;
char buffer[40];
static int count = 0;
char cphoto;
char cvideo;
char cflash;
char caction;
char cresult;
char ccamera;
int iphoto;
int ivideo;
int iflash;
int iaction;
int iresult;
int icamera;
String sreturn;

void setup() {

  pinMode(PIN_LED_PHOTO, OUTPUT);
  pinMode(PIN_LED_VIDEO, OUTPUT);
  //pinMode(PIN_LED_FLASH, OUTPUT);
  //pinMode(PIN_LED_ACTION, OUTPUT);

  // Switches Mode
  pinMode(PIN_BUTTON_PHOTO, INPUT_PULLUP);
  pinMode(PIN_BUTTON_VIDEO, INPUT_PULLUP);
  pinMode(PIN_BUTTON_FLASH, INPUT_PULLUP);

  // Joystick Mode
  //int valX = analogRead(JX_PHOTO_VIDEO);
  //int valY = analogRead(JY_FLASH);

  // Action Button
  pinMode(BUTTON_ACTION, INPUT_PULLUP);

  // Change Camera
  pinMode(BUTTON_CHANGE_CAMERA, INPUT_PULLUP);

  // App Mode
  pinMode(PIN_APP_MODE, INPUT_PULLUP);


  WRITE_PERI_REG(RTC_CNTL_BROWN_OUT_REG, 0);    //disable brownout detector   ds

  SerialBT.begin("Camera Remote Bluetooth");    //Bluetooth device name
  Serial.begin(115200);

  SerialBT.register_callback(callback);
  
  Serial.println("\nThe device started, now you can pair it with bluetooth!");
  
  delay(2000);
  WRITE_PERI_REG(RTC_CNTL_BROWN_OUT_REG, 1);    //enable brownout detector

}

void callback(esp_spp_cb_event_t event, esp_spp_cb_param_t *param) {
  if(event == ESP_SPP_SRV_OPEN_EVT){
    Serial.println("Client Connected");
    //digitalWrite(LED,HIGH);
    CONNECTED = true;
  }
 
  if(event == ESP_SPP_CLOSE_EVT ){
    Serial.println("Client disconnected");
    //digitalWrite(LED,LOW);
    CONNECTED = false;
  }
}


void loop() {

  currentTime = millis();
  if (modeChangedTime < 0) {
    modeChangedTime = currentTime;
  }

  if (CONNECTED) {
    while(SerialBT.available())
    {
        buffer[count] = SerialBT.read();
        count++;

        //Serial.println(buffer[count - 1]);
    }

    if(count>0){
      Serial.println(buffer);
  
      sreturn = buffer;
      if (sreturn.startsWith("photo")) {
  
        cresult = buffer[6];
        iresult = cresult - '0';
  
        if (iresult == 1) {
          digitalWrite(PIN_LED_PHOTO, HIGH);
        }
  
      } else if (sreturn.startsWith("video")) {
  
        cresult = buffer[6];
        iresult = cresult - '0';
  
        if (iresult == 1) {
          digitalWrite(PIN_LED_VIDEO, HIGH);
        } else if (iresult == 2) {
          MODE_VIDEO_RECORDING = !MODE_VIDEO_RECORDING;
          digitalWrite(PIN_LED_VIDEO, MODE_VIDEO_RECORDING);
        } 
        
      } else {
  
        cphoto = buffer[4];
        cvideo = buffer[6];
        cflash = buffer[8];
        caction = buffer[10];
        cresult = buffer[12];           //1: OK, 0:BAD
        //ccamera = buffer[14];           //0: back camera, 1: front camera
    
        iphoto = cphoto - '0';
        ivideo = cvideo - '0';
        iflash = cflash - '0';
        iaction = caction - '0';
        iresult = cresult - '0';
        //icamera = ccamera - '0';
  
        MODE_PHOTO = iphoto;
        MODE_VIDEO = ivideo;
        MODE_VIDEO_RECORDING = MODE_VIDEO;
        MODE_FLASH = iflash; 
  
        // return values from remote android phone
        digitalWrite(PIN_LED_PHOTO, MODE_PHOTO);
        digitalWrite(PIN_LED_VIDEO, MODE_VIDEO);
        //digitalWrite(PIN_LED_FLASH, MODE_FLASH);
  
      }
      
      count=0;
      memset(buffer,0,40);
    }
    
    // Set toggle app mode
    int appMode = digitalRead(PIN_APP_MODE);
    if (!appMode) {
      if (currentTime - modeChangedTime > 4000) {
        if (APP_MODE == 0) {
          APP_MODE = 1;
        } else {
          APP_MODE = 0;
        }
        ACTION_STATUS = 0;
        //      Serial.println(String(currentTime) + ", " + String(modeChangedTime) + ", " + String(APP_MODE));
        modeChangedTime = millis();
      }
    }
  
    if (APP_MODE == 1) {
      // Joystick Mode: not used 
      //    Serial.println("Joystick mode");
    } else if (APP_MODE == 0) {
      // Switches Mode
      //    Serial.println("Switches Mode");
  
      int photo = digitalRead(PIN_BUTTON_PHOTO);
      int video = digitalRead(PIN_BUTTON_VIDEO);
      int flash = digitalRead(PIN_BUTTON_FLASH);
      // Action Button for both modes
      int action = digitalRead(BUTTON_ACTION);
      // Change Camera
      int camera = digitalRead(BUTTON_CHANGE_CAMERA);
  
      if (ACTION_STATUS == 0) {
        
        if (!flash) {
          MODE_FLASH = !MODE_FLASH;
        } else if (!action) {
          // do nothing
        } else {
          // values from push button
          MODE_PHOTO = !photo;
          MODE_VIDEO = !video;    
        }
    
        if (!photo || !video || !flash || !action || !camera) {
    
          String actionString = "iot:" + String(MODE_PHOTO) + ":" + String(MODE_VIDEO) + ":" + String(MODE_FLASH) + ":" + String(!action) + ":" + String(!camera);

          if (CONNECTED) {

            SerialBT.println(actionString);
            SerialBT.write(13);
            SerialBT.write(10);
        
            ACTION_STATUS = 1;
      
            Serial.println(actionString);          
          }
  
          delay(50);     
        }      
        
      }
   
  
      if (ACTION_STATUS == 1) {
  
        if (photo && video && flash && action && camera) {
          ACTION_STATUS = 0;
          delay(10);      
        }
      
      }
      
    } // end of: (else if (APP_MODE == 0)

  } else { //end of: if (CONNECTED)

    // not connected
    digitalWrite(PIN_LED_PHOTO, LOW);
    digitalWrite(PIN_LED_VIDEO, LOW);
    
  }

  delay(10);
}
